<?php

namespace Drupal\eca_maestro\Plugin;

use Drupal\Core\TypedData\TypedDataInterface;

/**
 * Trait for ECA Maestro actions and conditions.
 */
trait MaestroTrait {

  /**
   * @param string $message
   * @param array $context
   *
   * @return void
   */
  public function info(string $message, array $context = []) {
    \Drupal::logger(ECA_MAESTRO_LOG_CHANNEL)->info($message, $context);
  }

  /**
   * @param string $message
   * @param array $context
   *
   * @return void
   */
  public function error(string $message, array $context = []) {
    \Drupal::logger(ECA_MAESTRO_LOG_CHANNEL)->error($message, $context);
  }

  /**
   * @param string $message
   * @param array $context
   *
   * @return void
   */
  public function warning(string $message, array $context = []) {
    \Drupal::logger(ECA_MAESTRO_LOG_CHANNEL)->warning($message, $context);
  }

  /**
   * @param string $label
   *
   * @return \Drupal\Core\StringTranslation\TranslatableMarkup
   */
  public function emptyMessage(string $label) {
    return $this->t('Empty @label.', ['@label' => $label]);
  }

  /**
   * @param string $label
   * @param mixed $value
   *
   * @return \Drupal\Core\StringTranslation\TranslatableMarkup
   */
  public function invalidMessage(string $label, mixed $value) {
    return $this->t('Invalid @label: @value',
      ['@label' => $label, '@value' => $value]);
  }

  /**
   * @param string $label
   *
   * @return \Drupal\Core\StringTranslation\TranslatableMarkup
   */
  public function undefinedMessage(string $label) {
    return $this->t('Undefined @label.', ['@label' => $label]);
  }

  /**
   * @return string|null
   */
  public function getAssignee($label = ECA_MAESTRO_ASSIGNEE): ?string {
    return $this->getConfigurationStringValue(
      ECA_MAESTRO_ASSIGNEE, $label, TRUE);
  }

  /**
   * @return int|null
   */
  public function getId($label = ECA_MAESTRO_ID_LABEL): ?int {
    return $this->getConfigurationIntValue(
      ECA_MAESTRO_ID, $label);
  }

  /**
   * @return int|null
   */
  public function getProcessId(): ?int {
    return $this->getConfigurationIntValue(
      ECA_MAESTRO_PROCESSID, ECA_MAESTRO_PROCESSID_LABEL);
  }

  /**
   * @return int|null
   */
  public function getQueueId(): ?int {
    return $this->getConfigurationIntValue(
      ECA_MAESTRO_QUEUEID, ECA_MAESTRO_QUEUEID_LABEL);
  }

  /**
   * @return int|null
   */
  public function getStartTask(): ?int {
    return $this->getConfigurationStringValue(
      ECA_MAESTRO_START, ECA_MAESTRO_START_LABEL);
  }

  /**
   * @return int|null
   */
  public function getTaskStatus(): ?int {
    return $this->getConfigurationIntValue(
      ECA_MAESTRO_STATUS, ECA_MAESTRO_STATUS_LABEL);
  }

  /**
   * @return string|null
   */
  public function getTemplateMachineName(): ?string {
    return $this->getConfigurationStringValue(
      ECA_MAESTRO_TEMPLATE, ECA_MAESTRO_TEMPLATE_LABEL, TRUE);
  }

  /**
   * @param bool $required
   *
   * @return string|null
   */
  public function getTokenName(bool $required = TRUE): ?string {
    return $this->getConfigurationStringValue(
      ECA_MAESTRO_TOKEN, ECA_MAESTRO_TOKEN_LABEL, $required);
  }

  /**
   * @return string|null
   */
  public function getType($label = ECA_MAESTRO_TYPE_LABEL): ?string {
    return $this->getConfigurationStringValue(
      ECA_MAESTRO_TYPE, $label, TRUE);
  }

  /**
   * @return int|null
   */
  public function getUserId(): ?int {
    return $this->getConfigurationIntValue(
      ECA_MAESTRO_USERID, ECA_MAESTRO_USERID_LABEL);
  }

  /**
   * @return string|null
   */
  public function getVariableName(): ?string {
    return $this->getConfigurationStringValue(
      ECA_MAESTRO_VARNAME, ECA_MAESTRO_VARNAME_LABEL, TRUE);
  }

  /**
   * @return int|null
   */
  public function getConfigurationIntValue(
    string $conf_name,
  string $conf_label): ?int {
    $value = $this->tokenServices->getOrReplace(
      $this->configuration[$conf_name]);
    if ($value) {
      if ($value instanceof TypedDataInterface) {
        $value = $value->getValue();
      }
      if (empty($value)) {
        $this->warning($this->emptyMessage($conf_label));
      }
      else {
        if (is_numeric($value)) {
          return intval($value);
        }
        else {
          $this->warning($this->invalidMessage($conf_label, $value));
        }
      }
    }
    else {
      $this->warning($this->undefinedMessage($conf_label));
    }
    return NULL;
  }

  /**
   * @return string|null
   */
  public function getConfigurationStringValue(
    string $conf_name,
  string $conf_label,
  bool $required = FALSE): ?string {
    $value = $this->tokenServices->getOrReplace(
      $this->configuration[$conf_name]);
    if ($value) {
      if ($value instanceof TypedDataInterface) {
        $value = $value->getString();
      }
      if ($required && empty($value)) {
        $this->warning($this->emptyMessage($conf_label));
      }
      else {
        return $value;
      }
    }
    else {
      if ($required) {
        $this->warning($this->undefinedMessage($conf_label));
      }
    }
    return NULL;
  }

  /**
   * @return mixed
   */
  public function getConfigurationMixedValue(
    string $conf_name,
  string $conf_label,
  bool $required = FALSE): mixed {
    $value = $this->tokenServices->getOrReplace(
      $this->configuration[$conf_name]);
    if ($value) {
      if ($value instanceof TypedDataInterface) {
        $value = $value->getValue();
      }
      if ($required && empty($value)) {
        $this->warning($this->emptyMessage($conf_label));
      }
      else {
        return $value;
      }
    }
    else {
      if ($required) {
        $this->warning($this->undefinedMessage($conf_label));
      }
    }
    return NULL;
  }

}
